/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */
import org.diacalc.mobile.components.*;
import com.sun.lwuit.*;
import com.sun.lwuit.events.*;
import com.sun.lwuit.layouts.*;
import com.sun.lwuit.table.*;


import org.diacalc.mobile.maths.*;
import javax.microedition.midlet.*;
import java.util.*;
import com.sun.lwuit.util.EventDispatcher;

public class CoefsForm  extends FormBase implements ActionListener{
    private DCJmobile master;
    
    private static final int N_CF_COMMAND = 30;
    private static final int E_CF_COMMAND = 31;
    private static final int D_CF_COMMAND = 32;

    private static final int YES_COMMAND = 36;
    private static final int NO_COMMAND = 37;

    private static final Command newCoefCommand = new Command("Новые коэф.", N_CF_COMMAND);
    private static final Command editCoefCommand = new Command("Изм. коэф.", E_CF_COMMAND);
    private static final Command delCoefCommand = new Command("Удал. коэф.", D_CF_COMMAND);
    

    private static final Command yesCommand = new Command("Да",YES_COMMAND);
    private static final Command noCommand = new Command("Нет",NO_COMMAND);

    private Table tblCoefs;
    
    protected void execute(MIDlet m,boolean refresh) {
        if (refresh){
            form.setLayout(new BorderLayout());

            master = (DCJmobile)m;
            TableLayout.setDefaultColumnWidth(25);

            tblCoefs = new Table(new MyTableModel(master.getCoefs()));
            //tblCoefs.setCellAlignment(Label.CENTER);
            
            form.addComponent(BorderLayout.CENTER, tblCoefs);
            form.addGameKeyListener(Display.GAME_FIRE,new ActionListener(){
                public void actionPerformed(ActionEvent ev){
                    if (tblCoefs.getSelectedColumn()>=0 && tblCoefs.getSelectedRow()>=0){
                        editCoef();
                    }
                }
            } );
            /*form.addPointerReleasedListener(new ActionListener(){
                public void actionPerformed(ActionEvent ev){
                    int x = ev.getX()-tblCoefs.getX();
                    int y = ev.getY()-tblCoefs.getY();
                    if (tblCoefs.getSelectedColumn()>=0 && tblCoefs.getSelectedRow()>=0 &&
                           x<tblCoefs.getWidth() && x>=0 &&
                           y<tblCoefs.getHeight() && y>=0){
                        System.out.println("clicked");
                    }
                    //System.out.println("x="+ev.getX()+" y="+ev.getY());
                    System.out.println(tblCoefs.getX()+" "+tblCoefs.getY()+" "+
                            tblCoefs.getWidth()+" "+tblCoefs.getHeight());
                    System.out.println(tblCoefs.getSelectedColumn() +" "+ tblCoefs.getSelectedRow() );
                }
            });*/

            form.addCommand(delCoefCommand);
            form.addCommand(editCoefCommand);
            form.addCommand(newCoefCommand);
            form.addCommandListener(this);
            form.setScrollable(true);
        }
    }
    public void actionPerformed(ActionEvent ev){
        int id = ev.getCommand().getId();
        switch(id){
            case E_CF_COMMAND: editCoef(); break;
            case D_CF_COMMAND: deleteCoef(); break;
            case N_CF_COMMAND: addCoefs(); break;
        }
        
    }
    private void editCoef(){
        int row = tblCoefs.getSelectedRow();
        if (row<0) return;
        CoefUnit cu = new CoefUnit( ((CoefUnit)((MyTableModel)tblCoefs.getModel())
                .getData().elementAt(row)) );
        
        Container c = new CoefEditor(cu,master.getUser());
        if (YES_COMMAND==Dialog.show("Измен. коэф-ты",
                c,
                new Command[]{yesCommand,noCommand} ).getId()){
                cu = ((CoefEditor)c).getCoefUnit();
                
                ((MyTableModel)tblCoefs.getModel()).setValueAt(row, 0, cu);
                master.setNeedSaveCoefs();
                if (master.getUser().isTimeSense()){
                    master.calcCoefsbyTime();
                    master.shouldRefreshMenu();
                }
        }
    }
    private void deleteCoef(){
        int row = tblCoefs.getSelectedRow();
        if (row<0) return;
        ((MyTableModel)tblCoefs.getModel()).removeDataAt(row);
        master.setNeedSaveCoefs();
        if (master.getUser().isTimeSense()){
            master.calcCoefsbyTime();
            master.shouldRefreshMenu();
        }
    }
    private void addCoefs(){
        CoefUnit cu = new CoefUnit();
        Container c = new CoefEditor(cu,master.getUser());
        if (YES_COMMAND==Dialog.show("Новые коэф-ты",
                c,
                new Command[]{yesCommand,noCommand} ).getId()){
                cu = ((CoefEditor)c).getCoefUnit();
                ((MyTableModel)tblCoefs.getModel()).addValue(cu);
                master.setNeedSaveCoefs();
                if (master.getUser().isTimeSense()){
                    master.calcCoefsbyTime();
                    master.shouldRefreshMenu();
                }
        }
    }
    public String getIconName(){
        return "coefs";
    }
    public String getName(){
        return "Коэфф.";
    }
    protected String getHelp() {
        return "Помощь";
    }
    class MyTableModel implements TableModel{
        private EventDispatcher dispatcher = new EventDispatcher();
        private Vector rows = new Vector();
        private Sugar s = new Sugar();

        public MyTableModel(Vector data){
            rows = data;
        }
        public Vector getData(){
            return rows;
        }
        public int getRowCount(){
            return rows.size();
        }
        public int getColumnCount(){
            return 4;
        }
        public java.lang.String getColumnName(int i){
            switch(i){
                case 0: return "Время";
                case 1: return "k1";
                case 2: return "k2";
                case 3: return "ЦЕИ";
                default: return "";
            }
        }
        public boolean isCellEditable(int row,
                       int column){
            return false;
        }
        public java.lang.Object getValueAt(int row,
                            int column){
            if (row>(rows.size()-1)) return null;
            CoefUnit cu = (CoefUnit)rows.elementAt(row);

            switch(column){
                case 0: return cu.getTimeString();
                case 1: return addZero(Str.round(cu.getFactors().getK1(Factors.DIRECT),2));
                case 2: return addZero(Str.round(cu.getFactors().getK2(),2));
                case 3: s.setValue(cu.getFactors().getK3());
                    if (master.getUser().isMmol()){
                        return addZero(Str.round(s.getSugar(master.getUser().isMmol(),
                            master.getUser().isPlasma()),2));
                    }else{
                        return ""+Str.round(s.getSugar(master.getUser().isMmol(),
                            master.getUser().isPlasma()),1);
                    }
                default: return "";
            }
        }
        private String addZero(float v){//0.00
            String s = "" + v;
            int p = s.indexOf(".");
            if (p<0) return s;
            //[ 0 . 0 ] p = 1 l = 3 l-p=2
            //[ 10.0  ] p = 2 l= 4 l-p=2
            //[ 10.00 ] p = 2 l =5 l-p=3
            if ((s.length()-p)<3) s += "0";
            return s;
        }
        private int addCoefUnit(CoefUnit cu){
            int i=0;
            for(;i<rows.size();i++){
                if ( ((CoefUnit)rows.elementAt(i)).getTime()>cu.getTime() ){
                    break;
                }
            }
            rows.insertElementAt(cu, i);
            //fireRowChanged(i);
            return i;
        }
        /*
          * непонятно нужен ли этот метод
          */
        /*private void fireRowChanged(int row){
            for(int j=0;j<getColumnCount();j++){
                fireDataChanged(j, row);
            }
        }*/
        public void addValue(Object o){
            addCoefUnit( (CoefUnit)o );
            //fireDataAdded( addCoefUnit( (CoefUnit)o ) );
        }
        public void removeDataAt(int row){
            if (row>(rows.size()-1)) return;
            rows.removeElementAt(row);
            //fireRowChanged(row);
        }
        public void setValueAt(int row,
                int column,
                java.lang.Object o){//Меняем весь объект сразу
            if(row>(rows.size()-1)) return;
            //System.out.println( ((CoefUnit)o).getTime() );
            //row старый индекс
            if ( ((CoefUnit)o).getTime()==
                    ((CoefUnit)rows.elementAt(row)).getTime() ){//Время не изменилось
                ((CoefUnit)rows.elementAt(row)).setFactors( ((CoefUnit)o).getFactors() );
                //fireRowChanged(row);
                
            }else{
                rows.removeElementAt(row);//удалили
                //fireRowChanged(row);
                addCoefUnit((CoefUnit)o);
            }
        }
        public void fireDataChanged(int col,int row){
            dispatcher.fireDataChangeEvent(col,row);
        }
        
        public void addDataChangeListener(DataChangedListener d){
            dispatcher.addListener(d);
        }
        public void removeDataChangeListener(DataChangedListener d){
            dispatcher.removeListener(d);
        }
    }
}
