/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */
package org.diacalc.mobile;

import com.sun.lwuit.Command;
import com.sun.lwuit.Component;
import com.sun.lwuit.Container;
import com.sun.lwuit.Form;
import com.sun.lwuit.Label;
import com.sun.lwuit.TextArea;
import com.sun.lwuit.events.ActionEvent;
import com.sun.lwuit.layouts.BorderLayout;
import com.sun.lwuit.geom.Dimension;
import com.sun.lwuit.Dialog;
import javax.microedition.midlet.*;
import com.sun.lwuit.Display;

/**
 * Base class for a demo contains common code for all demo pages
 *
 * @author Shai Almog
 */

public abstract class FormBase {
    protected Form form = null;
    
    /**
     * returns the name of the demo to display in the list
     */
    public abstract String getName();
    public abstract String getIconName();

    /**
     * Invoked by the main code to start the demo
     */
    public final void run(final Command backCommand, 
            DCJmobile m,boolean refresh) {
        if (form==null || refresh){
            if (form==null) refresh = true;
            form = new Form(getName()){
                protected Command showMenuDialog(Dialog menu) {
                    int marginLeft;// = (int) (BigMenuForm.this.getWidth() * 0.25f);
                    if (Display.getInstance().isTouchScreenDevice())    marginLeft = 0;
                    else        marginLeft = (int) (getWidth() * 0.25f);

                    int marginRight = 0;
                    if (isReverseSoftButtons()) {
                        marginRight = marginLeft;
                        marginLeft = 0;
                    }
                    int topMargin = getHeight() / 3;
                    return menu.show(topMargin, 0, marginLeft, marginRight, true);
                 }
            };
            form.addCommand(backCommand);
            form.addCommand(new Command("Справка") {
                public void actionPerformed(ActionEvent evt) {
                    Form helpForm = new Form("Справка");
                    helpForm.setLayout(new BorderLayout());
                    TextArea helpText = new TextArea(getHelpImpl(), 5, 10);
                    helpText.setEditable(false);
                    helpForm.setScrollable(false);
                    helpForm.addComponent(BorderLayout.CENTER, helpText);
                    Command c = new Command("Назад") {
                        public void actionPerformed(ActionEvent evt) {
                            form.show();
                        }
                    };
                    helpForm.addCommand(c);
                    helpForm.setBackCommand(c);
                    helpForm.show();
                }
            });
            form.addCommandListener(m);
            form.setBackCommand(backCommand);

        }
        execute(m,refresh);
        System.gc();//собираем мусор после пересоздания формы
        form.show();
    }

    /**
     * Returns the text that should appear in the help command
     */
    private String getHelpImpl() {
        return getHelp();
    }

    /**
     * Returns the text that should appear in the help command
     */
    protected String getHelp() {
        return ".help";
    }


    /**
     * The demo should place its UI into the given form
     */
    protected abstract void execute(MIDlet m,boolean refresh);


    public void cleanup() {
    }
}
