/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */

import com.sun.lwuit.*;
import com.sun.lwuit.layouts.*;
import com.sun.lwuit.events.*;
import org.diacalc.mobile.products.*;
import java.util.Vector;

import java.io.*;
import javax.microedition.io.*;
import javax.microedition.midlet.*;
import org.diacalc.mobile.maths.*;
import com.sun.lwuit.impl.midp.VirtualKeyboard;
import org.diacalc.mobile.components.InfoDialog;
import javax.microedition.io.file.*;
import javax.microedition.io.file.FileConnection;
import javax.microedition.io.file.FileSystemRegistry;

public class InternetForm extends FormBase implements ActionListener{
    private final static int DOWNLOAD_BASE_CL = 1;
    private final static int DOWNLOAD_MENU_CL = 2;
    private final static int UPLOAD_MENU_CL = 3;
    private final static int UPLOAD_BASE_CL = 4;
    private final static int UPLOAD_DIARY_CL = 5;

    private final static int DOWNLOAD_BASE = 31;
    private final static int UPLOAD_BASE = 32;
    private final static int DOWNLOAD_MENU = 33;
    private final static int UPLOAD_MENU = 34;
    private final static int UPLOAD_DIARY = 35;
    
    private static final int YES_COMMAND = 51;
    private static final int NO_COMMAND = 52;

    private final static Command cmdDownloadBase = new Command("Скачать базу",DOWNLOAD_BASE);
    private final static Command cmdUploadBase = new Command("Выгрузить базу",UPLOAD_BASE);
    private final static Command cmdDownloadMenu = new Command("Скачать меню",DOWNLOAD_MENU);
    private final static Command cmdUploadMenu = new Command("Выгрузить меню",UPLOAD_MENU);
    private final static Command cmdUploadDiary = new Command("Выгрузить дневник",UPLOAD_DIARY);
    
    private static final Command yesCommand = new Command("Да",YES_COMMAND);
    private static final Command noCommand = new Command("Нет",NO_COMMAND);

    private final static String DEFAULT_SERVER = "http://diacalc.org/dbwork/";
    private final static String SCRIPT_NAME = "server.php";

    private TextField fldLogin;
    private TextField fldPass;
    private TextField fldServer;
    private Button btnBase;
    
    private DCJmobile master;
    private boolean snackLoaded = false;
    
    private String verMsg;
    private boolean shouldCheckVer = true;

    public String getName() {
        return "Интернет";
    }
    public String getIconName(){
        return "inet";
    }
    protected String getHelp() {
        return "В этом окне можно загрузить и выгрузить базу продуктов с/на " +
                "сервер, скачать/выгрузить меню или сохранить на сервер записи " +
                "дневника.\n" +
                "Для использования этой возможности необходимо иметь учетную " +
                "запись программы DiaCalcJ.\n" +
                "Для ее получения необходимо зарегистрироваться по адресу " +
                "http://diacalc.org/dbwork/";
    }
    public void actionPerformed(ActionEvent ev){
        int id = ev.getCommand().getId();
        switch (id){
            case DOWNLOAD_BASE: doNetworking(DOWNLOAD_BASE_CL); break;
            case DOWNLOAD_MENU: doNetworking(DOWNLOAD_MENU_CL); break;
            case UPLOAD_MENU:   doNetworking(UPLOAD_MENU_CL);   break;
            case UPLOAD_BASE:   doNetworking(UPLOAD_BASE_CL);   break;
            case UPLOAD_DIARY:  doNetworking(UPLOAD_DIARY_CL);  break;
        }
    }
    private ActionListener saveUser = new ActionListener(){
        public void actionPerformed(ActionEvent ev){
            saveUserInput();
        }
    };
    
    class DownloaderBase implements NetWorker{
        private boolean error = false;
        private String msg = "";

        public boolean isError(){
            return error;
        }
        public void work(){
            String request = addKeyValue("","login",fldLogin.getText());
            request = addKeyValue(request,"pass",fldPass.getText());
            request = addKeyValue(request,"action","get mobile base");
            request = addKeyValue(request,"ok","ok");

            DoingPOST post = new DoingPOST(checkServerPath(),request);
            String res = post.getAnswer();

            if (res.length()>0 && res.endsWith("<ok>")){
                //String msg = "";
                Vector groups = null;
                try{
                    groups = Str.getFirstTagged(res, "groups");
                }catch(Exception ex){
                    msg = ex.getMessage();
                    error = true;
                    return;
                }
                
                Vector prods = null;
                try{
                    prods = Str.getFirstTagged(res, "prods");
                }catch(Exception ex){
                    msg += ex.getMessage();
                    error = true;
                    return;
                }
                
                Vector groupsV = parseGroups((String)groups.elementAt(0));
                Vector prodsV = parseProducts((String)prods.elementAt(0));
                
                if (groupsV.isEmpty() || prodsV.isEmpty()){
                    msg += "Загруженная база не полная\nГрупп:"+groupsV.size()+
                            " продуктов:"+prodsV.size()+"\nБаза продуктов не обновлена";
                    error = true;
                    return;
                }else{
                    master.setLoadedBase(groupsV, prodsV);
                    msg = "База успешно загружена";
                }
            }
            else{
                error=post.isError();
                msg += "Произошла ошибка при получении данных\n"+post.getErrorMessage();
            }
        }
        
        public void showMessage(){
            TextArea txt = new TextArea(Str.replace(msg,"<br>","\n"));
            txt.setEditable(false);
            Dialog.show(error?"Ошибка":"Сообщение",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_INFO,
                            null);
        }
    }
    class UploaderBase implements NetWorker{
        private boolean error = false;
        private String msg = "";

        public boolean isError(){
            return error;
        }
        public void work(){
            if (master.getGroups().isEmpty() || master.getProds().isEmpty()){
                error = true;
                msg = "У Вас пустая база продуктов.\nНечего выгружать.";
                return;
            }

            Vector exGr = new Vector();
            Vector exPr = new Vector();
            //Добавляем продукты и группы
            for(int p=0;p<master.getProds().size();p++){
                if ( ((ProductInBase)master.getProds().elementAt(p)).isMobile() ){
                    exPr.addElement( master.getProds().elementAt(p) );
                    boolean found = false;
                    for(int g=0;g<exGr.size();g++){
                        if ( ((ProductInBase)master.getProds().elementAt(p)).getOwner()==
                                ((ProductGroup)exGr.elementAt(g)).getId() ){
                            found = true;
                            break;
                        }
                    }
                    if (!found){
                        //Добавляем группу
                        for(int g=0;g<master.getGroups().size();g++){
                            if ( ((ProductInBase)master.getProds().elementAt(p)).getOwner()==
                                ((ProductGroup)master.getGroups().elementAt(g)).getId() ){
                                exGr.addElement( master.getGroups().elementAt(g) );
                                break;
                            }
                        }
                    }
                }
            }
            //есть что отправлять
            if (exPr.isEmpty()){
                error = true;
                msg = "Нет продуктов добавленных или изменных в телефоне.\nНечего выгружать.";
                return;
            }

            String query = addKeyValue("","login",fldLogin.getText());
            query = addKeyValue(query,"pass",fldPass.getText());
            query = addKeyValue(query,"action","send m_base");//Новое имя


            for(int i=0;i<exGr.size();i++){
                query = addKeyValue(query,"grids[]",""+((ProductGroup)exGr.elementAt(i)).getId());
                query = addKeyValue(query,"grnames[]",((ProductGroup)exGr.elementAt(i)).getName());
            }

            //теперь посылаем продукты из этой группы
            for(int p=0;p<exPr.size();p++){
                ProductInBase pr = (ProductInBase)exPr.elementAt(p);
                    query = addKeyValue(query,"prowners[]",
                            ""+pr.getOwner());
                    query = addKeyValue(query,"prnames[]",
                            pr.getName());
                    query = addKeyValue(query,"prprots[]",""+
                            pr.getProt());
                    query = addKeyValue(query,"prfats[]",""+
                            pr.getFat());
                    query = addKeyValue(query,"prcarbs[]",""+
                            pr.getCarb());
                    query = addKeyValue(query,"prgis[]",""+
                            pr.getGi());
                    query = addKeyValue(query,"prweights[]",""+
                            pr.getWeight());
                    query = addKeyValue(query,"prusages[]",""+
                            pr.getUsage());
            }

            query = addKeyValue(query,"ok","ok");

            DoingPOST post = new DoingPOST(checkServerPath(),query);
            String res = post.getAnswer();
            
            if (res.length()>0 && res.endsWith("<ok>")){
                msg = "База продуктов успешно выгружена на сервер";
            }else{
                error = post.isError();
                msg += "Произошла ошибка при выгрузке данных\n"+post.getErrorMessage();
            }

        }
        public void showMessage(){
            TextArea txt = new TextArea(Str.replace(msg, "<br>", "\n"));
            txt.setEditable(false);
            Dialog.show(error?"Ошибка":"Сообщение",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_INFO,
                            null);
        }
    }

    class DownloaderMenu implements NetWorker{
        private boolean error = false;
        private String msg = "";
        private Vector menuV;

        public boolean isError(){
            return error;
        }
        public void work(){
            String request = addKeyValue("","login",fldLogin.getText());
            request = addKeyValue(request,"pass",fldPass.getText());
            request = addKeyValue(request,"action","get menu");
            request = addKeyValue(request,"ok","ok");

            DoingPOST post = new DoingPOST(checkServerPath(),request);
            String res = post.getAnswer();

            if (res.length()>0 && res.endsWith("<ok>")){
                Vector menu = null;
                try{
                    menu = Str.getFirstTagged(res, "menu");
                }catch(Exception ex){
                    error = true;
                    msg = ex.getMessage();
                    return;
                }
                Vector coefs = null;
                try{
                    coefs = Str.getFirstTagged(res, "coefs");
                }catch(Exception ex){
                    error = true;
                    msg = ex.getMessage();
                    return;
                }
                menuV = new Vector();

                menuV = parseMenu((String)menu.elementAt(0));
                master.setUser( parseCoefs((String)coefs.elementAt(0)) );

                msg = "Меню успешно загружено\nсуществующее меню будет очищено";
                if (snackLoaded) msg += "\nЗагружен перекус, он будет помещен в общий список";
            }
            else{
                error=post.isError();
                msg += "Произошла ошибка при получении данных\n"+post.getErrorMessage();
            }
        }
        
        public void showMessage(){
            msg = Str.replace(msg, "<br>", "\n");
            if (error){
                Dialog.show("Ошибка", msg, "OK", null);
            }else{
                TextArea txt = new TextArea(msg);
                txt.setEditable(false);
                if (Dialog.show("Сообщение",
                    txt,
                    new Command[]{yesCommand,noCommand} ).getId()==YES_COMMAND){
                    master.setMenu(menuV);
                    snackLoaded = false;
                }
            }
        }
    }
    class UploaderMenu implements NetWorker{
        private boolean error = false;
        private String msg = "";

        public boolean isError(){
            return error;
        }
        public void work(){
            if (master.getMenu().isEmpty()){
                error = true;
                msg = "Нечего выгружать\nМеню пустое";
                return;
            }
            String query = addKeyValue("","login",fldLogin.getText());
            query = addKeyValue(query,"pass",fldPass.getText());
            query = addKeyValue(query,"action","send menu");//Посылаем обычным способом

            query = addKeyValue(query,"k1",""+master.getUser().getFactors().getK1Value()*10/master.getUser().getFactors().getBEValue());//us.getFactors().getK1Value()*10/us.getFactors().getBEValue()
            query = addKeyValue(query,"k2",""+master.getUser().getFactors().getK2());
            query = addKeyValue(query,"k3",""+master.getUser().getFactors().getK3() );
            query = addKeyValue(query,"sh1",""+master.getUser().getS1());
            query = addKeyValue(query,"sh2",""+master.getUser().getS2());

            for(int i=0;i<master.getMenu().size() && !error;i++){
                ProductInMenu pr = (ProductInMenu)master.getMenu().elementAt(i);
                query = addKeyValue(query,"names[]", pr.getName());
                query = addKeyValue(query,"prots[]", ""+pr.getProt());
                query = addKeyValue(query,"fats[]", ""+pr.getFat());
                query = addKeyValue(query,"carbs[]", ""+pr.getCarb());
                query = addKeyValue(query,"gis[]", ""+pr.getGi());
                query = addKeyValue(query,"weights[]", ""+pr.getWeight());
                query = addKeyValue(query,"issnacks[]", "0");
            }
            query = addKeyValue(query,"ok","ok");

            DoingPOST post = new DoingPOST(checkServerPath(),query);
            String res = post.getAnswer();
            
            if (res.endsWith("<ok>")){
                msg = "Меню успешно выгружено на сервер";
            }
            else{
                error = post.isError();
                msg = "Произошли ошибки при выгрузке меню\n"+post.getErrorMessage();
            }
        }
        public void showMessage(){
            TextArea txt = new TextArea(Str.replace(msg,"<br>","\n"));
            txt.setEditable(false);
            Dialog.show(error?"Ошибка":"Сообщение",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_INFO,
                            null);
        }
    }
    class UploaderDiary implements NetWorker{
        private boolean error = false;
        private String msg = "";

        public boolean isError(){
            return error;
        }
        public void work(){
            if (master.getDiary().isEmpty()){
                error = true;
                msg = "Нет записей для выгрузки";
                return;
            }
            Vector sugars  = new Vector();
            Vector menus = new Vector();
            Vector comments = new Vector();

            for(int i=0;i<master.getDiary().size();i++){
                switch (((DiaryUnit)master.getDiary().elementAt(i)).getType() ){
                    case DiaryUnit.SUGAR  : sugars.addElement(master.getDiary().elementAt(i));  break;
                    case DiaryUnit.MENU   : menus.addElement(master.getDiary().elementAt(i));   break;
                    case DiaryUnit.COMMENT: comments.addElement(master.getDiary().elementAt(i));break;
                }

            }

            String pre_query = addKeyValue("","login",fldLogin.getText());
            pre_query = addKeyValue(pre_query,"pass",fldPass.getText());
            pre_query = addKeyValue(pre_query,"action","send m_diary");
            
            String query = pre_query;
            //Записи о сахарах
            for(int i=0;i<sugars.size();i++){
                DiaryUnit du = (DiaryUnit)sugars.elementAt(i);
                query = addKeyValue(query,"duStime[]",""+
                        (du.getTime()+master.getUser().getTimeDiary()*1000*60*60));
                query = addKeyValue(query,"duSsugar[]",""+du.getSh1());
                query = addKeyValue(query,"duScomm[]",""+du.getComment());
            }
            query = addKeyValue(query,"ok","ok");

            DoingPOST post = new DoingPOST(checkServerPath(),query);
            String res = post.getAnswer();

            if (!res.endsWith("<ok>")){
                error = post.isError();
                msg += "Не удалось выгрузить записи о СК\n"+post.getErrorMessage();
            }

            query = pre_query;
            //Коментарии
            for(int i=0;i<comments.size();i++){
                DiaryUnit du = (DiaryUnit)comments.elementAt(i);
                query = addKeyValue(query,"duCtime[]",""+
                        (du.getTime()+master.getUser().getTimeDiary()*1000*60*60));
                query = addKeyValue(query,"duCcomm[]",""+du.getComment());
            }
            query = addKeyValue(query,"ok","ok");

            post = new DoingPOST(checkServerPath(),query);
            res = post.getAnswer();

            if (!res.endsWith("<ok>")){
                error = post.isError();
                msg += "Не удалось выгрузить коментарии\n"+post.getErrorMessage();
            }

            boolean prods_error = false;
            //Начинаем сложные записи
            for(int i=0;i<menus.size();i++){
                query = pre_query;
                DiaryUnit du = (DiaryUnit)menus.elementAt(i);

                query = addKeyValue(query,"duMid[]",""+du.getId());
                query = addKeyValue(query,"duMtime[]",""+
                        (du.getTime()+master.getUser().getTimeDiary()*1000*60*60));
                query = addKeyValue(query,"duMcomm[]",du.getComment());
                query = addKeyValue(query,"duMsh1[]",""+du.getSh1());
                query = addKeyValue(query,"duMsh2[]",""+du.getSh2());
                query = addKeyValue(query,"duMk1[]",""+du.getFactors().getK1Value()*10/master.getUser().getFactors().getBEValue()); //us.getFactors().getK1Value()*10/us.getFactors().getBEValue()
                query = addKeyValue(query,"duMk2[]",""+du.getFactors().getK2());
                query = addKeyValue(query,"duMk3[]",""+du.getFactors().getK3());
                query = addKeyValue(query,"duMdose[]",""+du.getDose());
                query = addKeyValue(query,"duMprot[]",""+du.getProduct().getProt());
                query = addKeyValue(query,"duMfat[]",""+du.getProduct().getFat());
                query = addKeyValue(query,"duMcarb[]",""+du.getProduct().getCarb());
                query = addKeyValue(query,"duMgi[]",""+du.getProduct().getGi());
                query = addKeyValue(query,"duMweight[]",""+du.getProduct().getWeight());
                
                //Продукты
                for(int p=0;p<master.getDiaryRecs().size();p++){
                    ProductInMenu pr = (ProductInMenu)master.getDiaryRecs().elementAt(p);
                    if (du.getId()==pr.getOwner()){

                        query = addKeyValue(query,"drowner[]",""+pr.getOwner());
                        query = addKeyValue(query,"drname[]",pr.getName());
                        query = addKeyValue(query,"drprot[]",""+pr.getProt());
                        query = addKeyValue(query,"drfat[]",""+pr.getFat());
                        query = addKeyValue(query,"drcarb[]",""+pr.getCarb());
                        query = addKeyValue(query,"drgi[]",""+pr.getGi());
                        query = addKeyValue(query,"drweight[]",""+pr.getWeight());
                    }
                }
                query = addKeyValue(query,"ok","ok");
                post = new DoingPOST(checkServerPath(),query);
                res = post.getAnswer();

                if (!res.endsWith("<ok>")){
                    prods_error = true;
                }
            }
            if (prods_error){
                error = true;
                msg += "Не удалось выгрузить одну или несколько записей о меню\n"+
                        post.getErrorMessage();
            }

            if (!error){//(res.endsWith("<ok>")){
                msg = "Записи дневника успешно выгружены на сервер";
                master.getDiary().removeAllElements();
                master.getDiaryRecs().removeAllElements();
                master.setNeedSaveDiary();
            }
            
        }

        public void showMessage(){
            TextArea txt = new TextArea(Str.replace(msg, "<br>", "\n"));
            txt.setEditable(false);
            Dialog.show(error?"Ошибка":"Сообщение",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_INFO,
                            null);
        }
    }
    private void saveRes(String in){
        try{
            byte [] data = in.getBytes();

            FileConnection fc = (FileConnection) Connector.open("file:///e:/other/diary.txt",Connector.READ_WRITE);
            if (!fc.exists()) {
                fc.create();
            }
            //OutputStreamWriter os = new OutputStreamWriter(fc.openOutputStream(),"UTF8");
            OutputStream os = fc.openOutputStream();
            os.write(data);
            //os.write(log.getText());
            //os.flush();

            os.close();
            fc.close();

        }catch(Exception ex){
            Dialog.show("Ошибка",
                new TextArea("Произошла ошибка при попытке записи файла\n"+
                    ex.getMessage(),3,20),
                new Command [] {new Command("Ok")},
                Dialog.TYPE_ERROR, null);
        }
    }
    private String checkVersion(){
        String query = addKeyValue("","login",fldLogin.getText());
        query = addKeyValue(query,"pass",fldPass.getText());
        query = addKeyValue(query,"action","check version");
        query = addKeyValue(query,"pName","DCJM");
        query = addKeyValue(query,"pVer",""+DCJmobile.PROGRAM_VERSION);
        query = addKeyValue(query,"ok","ok");

        DoingPOST post = new DoingPOST(checkServerPath(),query);
        String res = post.getAnswer();

        if (res.endsWith("<ok>")){//Нормальный ответ
            return Str.replace(res, "<ok>", "");
        }else{
            return "Произошла ошибка при проверке версии, сообщите разработчику\n"+
                    post.getErrorMessage();
        }
    }
    private void doNetworking(int action){
        InfoDialog infoDlg = new InfoDialog();
        final NetWorker worker;
        switch(action){
            case DOWNLOAD_BASE_CL:  worker = new DownloaderBase();  break;
            case DOWNLOAD_MENU_CL:  worker = new DownloaderMenu();  break;
            case UPLOAD_MENU_CL:    worker = new UploaderMenu();    break;
            case UPLOAD_BASE_CL:    worker = new UploaderBase();    break;
            case UPLOAD_DIARY_CL:   worker = new UploaderDiary();   break;
            default: worker = null;
        }

        infoDlg.showModeless();

        verMsg = "";
        //А сама загрузка должна быть в отдельном потоке
        Display.getInstance().invokeAndBlock(new Runnable() {
            public void run() {
                // perform IO operation...
                worker.work();
                //тут нужно проверить актуальность версии и сообщение с сервера
                if (shouldCheckVer && !worker.isError()){
                    verMsg = checkVersion();
                    shouldCheckVer = false;
                }
            }
        });
        infoDlg.dispose();

        worker.showMessage();
        if (verMsg.length()>0){
            TextArea txt = new TextArea(verMsg);
            txt.setEditable(false);
            Dialog.show("Обновление",
                            txt,
                            new Command[] {new Command("Ok")},
                            Dialog.TYPE_INFO,
                            null);
        }
    }
    private User parseCoefs(String st){
        Vector lines = Str.split(st.trim(),"<br>");
        for(int i=lines.size();i>0;i--){
            if ( ((String)lines.elementAt(i-1)).trim().length()==0 ){
                lines.removeElementAt(i-1);
            }
        }
        User user = master.getUser();
        if (lines.size()==1){
            Vector vl = Str.split( (String)lines.elementAt(0), " ");
            if (vl.size()==5){
                try{
                    float k1 = Float.parseFloat((String)vl.elementAt(0)) *
                            user.getFactors().getBE(Factors.DIRECT) / 10f;
                    user.getFactors().setK1Value(k1);
                    user.getFactors().setK2( Float.parseFloat((String)vl.elementAt(1)) );
                    user.getFactors().setK3( Float.parseFloat((String)vl.elementAt(2)) );

                    user.setS1(Float.parseFloat((String)vl.elementAt(3)));
                    user.setS2(Float.parseFloat((String)vl.elementAt(4)));
                }catch(Exception ex){}
            }
        }

        return user;
    }

    private Vector parseMenu(String st){
        Vector arr = new Vector();
        Vector lines = Str.split(st.trim(),"<br>");
        for(int i=0;i<lines.size();i++){
          if (((String)lines.elementAt(i)).trim().length()>0){
            Vector param = Str.split( ((String)lines.elementAt(i)).trim(),
                    " ");
            int gi;
            float w,p,c,f;
            if (param.size()>1){
                try{
                    snackLoaded = snackLoaded ||
                            Integer.parseInt((String)param.elementAt(param.size()-1))!=0;
                    w = Float.parseFloat(
                        ((String)param.elementAt(param.size()-2)) );
                    gi = Integer.parseInt((String)param.elementAt(param.size()-3));
                    c = Float.parseFloat(
                        ((String)param.elementAt(param.size()-4)) );
                    f = Float.parseFloat(
                        ((String)param.elementAt(param.size()-5)) );
                    p = Float.parseFloat(
                        ((String)param.elementAt(param.size()-6)));
                }catch(Exception ex){
                    gi = 0;
                    w = p = f = c = 0f;
                }
                String name ="";
                for(int j=0;j<(param.size()-6);j++){
                    name += (String)param.elementAt(j) + " ";
                }
                name = name.trim();
                arr.addElement(new ProductInMenu(name,p,f,c,gi,w,-1));
            }
          }
        }
        return arr;
    }
    private Vector parseGroups(String st){
        Vector arr = new Vector();
        Vector lines = Str.split(st.trim(),"<br>");
        for(int i=0;i<lines.size();i++){
          if (((String)lines.elementAt(i)).trim().length()>0){
            Vector param = Str.split( ((String)lines.elementAt(i)).trim(),
                    " ");
            int id;
            if (param.size()>1){
                try{
                    id = Integer.parseInt((String)param.elementAt(param.size()-1));
                }catch(Exception ex){
                    //ex.printStackTrace();
                    id = 0;
                }
                String name ="";
                for(int j=0;j<(param.size()-1);j++){
                    name += (String)param.elementAt(j) + " ";
                }
                name = name.trim();
                arr.addElement(new ProductGroup(name,id));
            }
          }
        }
        return arr;
    }
    private Vector parseProducts(String st){
        Vector arr = new Vector();
        Vector lines = Str.split(st.trim(),"<br>");
        for(int i=0;i<lines.size();i++){
          if (((String)lines.elementAt(i)).trim().length()>0){
            Vector param = Str.split( ((String)lines.elementAt(i)).trim(),
                    " ");
            int gi,owner,usage;
            float c,f,p;
            if (param.size()>6){
                try{
                    owner = Integer.parseInt((String)param.elementAt(param.size()-1));
                    usage = Integer.parseInt((String)param.elementAt(param.size()-2));
                    gi = Integer.parseInt((String)param.elementAt(param.size()-3));
                    c = Float.parseFloat(
                    ((String)param.elementAt(param.size()-4)) );
                    f = Float.parseFloat(
                    ((String)param.elementAt(param.size()-5)) );
                    p = Float.parseFloat(
                    ((String)param.elementAt(param.size()-6)) );
                }catch(Exception ex){
                    //ex.printStackTrace();
                    owner = usage = gi = 0;
                    c = f = p = 0f;
                }
                String name ="";
                for(int j=0;j<(param.size()-6);j++){
                    name += (String)param.elementAt(j) + " ";
                }
                name = name.trim();
                arr.addElement(new ProductInBase(name,p,f,c,gi,100f,false,owner,usage));
            }
          }
        }
        return arr;
    }

    private String addKeyValue(String query, String key, String value){
        String res;
        if (query==null){
            res = "";
        }
        else res = query;
        if (res.length()>0){
            res += "&";
        }
        //if (value.length()==0) value = "===";
        res += key + "=" + Str.URLencode(value);

        return res;
    }
    private String checkServerPath(){
        String url = fldServer.getText();
        if (url.length()==0){
            url = DEFAULT_SERVER;
            fldServer.setText(url);
        }
        else if (!url.endsWith("/")){
            url += "/";
            fldServer.setText(url);
        }
        url += SCRIPT_NAME;
        return url;
    }
    
    protected void execute(MIDlet m,boolean refresh) {
        master = (DCJmobile)m;
        if (refresh){
            Label lblLogin = new Label("login:");
            fldLogin = new TextField(master.getUser().getLogin());//"1+2","Abc","abc","ABC"
            fldLogin.setInputModeOrder(new String[]{"Abc","abc","ABC","123"});
            fldLogin.setInputMode("Abc");
            fldLogin.addActionListener(saveUser);
            fldLogin.setLabelForComponent(lblLogin);

            Label lblPass = new Label("password:");
            fldPass = new TextField(master.getUser().getPass());
            fldPass.setConstraint(TextField.PASSWORD);
            fldPass.setInputModeOrder(new String[]{"abc","ABC","123"});
            fldPass.setInputMode("abc");
            fldPass.addActionListener(saveUser);
            fldPass.setLabelForComponent(lblPass);

            Label lblServer = new Label("server:");
            fldServer = new TextField(DEFAULT_SERVER);
            fldServer.setConstraint(TextField.URL);
            fldServer.setInputModeOrder(new String[]{"abc","ABC","123"});
            fldServer.setInputMode("abc");
            fldServer.addActionListener(saveUser);
            fldServer.setLabelForComponent(lblServer);

            btnBase = createButton(cmdDownloadBase);
            //Создаем кнопку, что бы на нее можно было передать фокус
            
            form.setLayout(new BoxLayout(BoxLayout.Y_AXIS));

            //Задумывал разместить кнопки в два столбца, но пока не придумал как
            Container up = new Container(new BoxLayout(BoxLayout.Y_AXIS));
            up.addComponent(lblServer);
            up.addComponent(fldServer);
            up.addComponent(lblLogin);
            up.addComponent(fldLogin);
            up.addComponent(lblPass);
            up.addComponent(fldPass);

            form.addComponent(up);

            //Container down = new Container();

            form.addComponent(btnBase);
            form.addComponent(createButton(cmdUploadBase));
            form.addComponent(createButton(cmdDownloadMenu));
            form.addComponent(createButton(cmdUploadMenu));
            form.addComponent(createButton(cmdUploadDiary));


            if (Display.getInstance().isTouchScreenDevice()){
                VirtualKeyboard vk = new VirtualKeyboard();
                vk.setInputModeOrder(new String[]{VirtualKeyboard.QWERTY_MODE,VirtualKeyboard.NUMBERS_SYMBOLS_MODE});
                VirtualKeyboard.bindVirtualKeyboard(fldPass, vk);
                VirtualKeyboard.bindVirtualKeyboard(fldLogin, vk);
                VirtualKeyboard.bindVirtualKeyboard(fldServer, vk);

                vk.addCommandListener(new ActionListener(){
                    public void actionPerformed(ActionEvent ev){
                        if (ev.getCommand().getCommandName().equals("Ok")){
                            Display.getInstance().callSerially(new Runnable(){
                                public void run(){
                                    saveUserInput();
                                }
                            });
                        }
                    }
                });

            }
        }
        btnBase.requestFocus();
    }
    private void saveUserInput(){
        master.getUser().setLogin(fldLogin.getText());
        master.getUser().setPass(fldPass.getText());
        master.getUser().setServer(fldServer.getText());
    }
    private Button createButton(Command c){
        Button btn = new Button(c);
        btn.setAlignment(Button.CENTER);
        btn.addActionListener(this);
        return btn;
    }

    interface NetWorker{
        public void work();
        public void showMessage();
        public boolean isError();
    }
}
