/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile.components;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */
import com.sun.lwuit.*;
import com.sun.lwuit.events.*;
import com.sun.lwuit.layouts.*;
import org.diacalc.mobile.maths.*;
import com.sun.lwuit.impl.midp.VirtualKeyboard;
import org.diacalc.mobile.DCJmobile;

public class CoefEditor extends Container implements ActionListener{
    private MaskTextField time;
    private MaskTextField fldK1;
    private MaskTextField fldK2;
    private TextField fldK3;
    private CoefUnit cu;
    private User us;

    public CoefEditor(CoefUnit cf,User u){
        super(new BoxLayout(BoxLayout.Y_AXIS));
        us = u;
        cu = cf;
        time = new MaskTextField("##:##");
        time.setValidator(new TimeValidator());
        time.setValue(new Integer(cu.getTime()));
        //time.addActionListener(this);

        fldK1 = new MaskTextField("#.##");
        FloatValidator fl = new FloatValidator();
        fldK1.setValidator(fl);
        fldK1.setValue(new Float(cu.getFactors().getK1(Factors.DIRECT)));
        fldK1.addActionListener(this);

        fldK2 = new MaskTextField("#.##");
        fldK2.setValidator(fl);
        fldK2.setValue(new Float(cu.getFactors().getK2()));
        fldK2.addActionListener(this);

        fldK3 = new TextField();
        fldK3.setInputModeOrder(new String[]{"1+2"});
        fldK3.setInputMode("1+2");
        fldK3.setConstraint(TextField.DECIMAL);
        Sugar s = new Sugar(cu.getFactors().getK3());
        if (us.isMmol()){
            fldK3.setText(""+Str.round(
                s.getSugar(us.isMmol(),
                    us.isPlasma()),2));
        }else{
            fldK3.setText(""+Str.round(
                s.getSugar(us.isMmol(),
                    us.isPlasma()),1));
        }
        fldK3.addActionListener(this);
        
        addComponent(Str.createPair("Время",time,60));
        addComponent(Str.createPair("k1",fldK1,60));
        addComponent(Str.createPair("k2",fldK2,60));
        addComponent(Str.createPair("ЦЕИ",fldK3,60));

        if (Display.getInstance().isTouchScreenDevice()){
            VirtualKeyboard vk = DCJmobile.createNumbersKeyboard();
            VirtualKeyboard.bindVirtualKeyboard(time, vk);
            VirtualKeyboard.bindVirtualKeyboard(fldK1, vk);
            VirtualKeyboard.bindVirtualKeyboard(fldK2, vk);
            VirtualKeyboard.bindVirtualKeyboard(fldK3, vk);
            vk.addCommandListener(new ActionListener(){
                public void actionPerformed(ActionEvent ev){
                    if (ev.getCommand().getCommandName().equals("Ok")){
                       Display.getInstance().callSerially(new Runnable() {
                            public void run() {
                                time.setValue( time.getValue() );
                                fldK1.setValue(fldK1.getValue() );
                                fldK2.setValue(fldK2.getValue() );
                            }
                       });
                    }
                }
            });
        }
        fldK1.getValue();//Заполняем маску
        fldK2.getValue();
    }
    public void actionPerformed(ActionEvent ev){
        Str.parseString( (TextField)ev.getSource() , 2);
    }
    public CoefUnit getCoefUnit(){
        cu.setTime( ((Integer)time.getValue()).intValue() );
        cu.getFactors().setK1XE(((Float)fldK1.getValue()).floatValue(),
                us.getFactors().getBE(Factors.DIRECT),
                Factors.DIRECT);
        cu.getFactors().setK2( ((Float)fldK2.getValue()).floatValue() );
        Sugar ouv = new Sugar();
        ouv.setSugar(Str.parseString(fldK3, 2), us.isMmol(), us.isPlasma());
        cu.getFactors().setK3(ouv.getValue());
        return cu;
    }
}
