/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */

package org.diacalc.mobile.components;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */

import com.sun.lwuit.*;
import com.sun.lwuit.plaf.UIManager;
import com.sun.lwuit.layouts.*;
import com.sun.lwuit.list.*;
import com.sun.lwuit.events.*;
import org.diacalc.mobile.products.ProductGroup;


public class HorizontalList extends Container implements ActionListener{
    private ListModel model;
    private AnimatedLabel anLabel;
    private Button left;
    private Button right;
    private boolean searchMode = false;
    private ProductGroup gr = new ProductGroup("",-1);
    public void setSearch(boolean v){
        searchMode = v;
        if (v){//включаем режим поиска
            anLabel.setText("Поиск");
        }
        else{
            int in = model.getSelectedIndex();
            anLabel.setText(""+(in+1)+". "+model.getItemAt(in).toString());
        }
    }
    public HorizontalList(ListModel model){
        super(new BorderLayout());
        this.model = model;
        left = new Button("<");
        right = new Button(">");
        left.addActionListener(this);
        right.addActionListener(this);
        left.setFocusable(false);
        right.setFocusable(false);
        if (!Display.getInstance().isTouchScreenDevice()){
            left.setUIID("ArrowButton");
            right.setUIID("ArrowButton");
        }
        else{
            int w = Display.getInstance().getDisplayWidth();
            left.setPreferredW(w*12/100);
            right.setPreferredW(w*12/100);
            left.setAlignment(Button.CENTER);
            right.setAlignment(Button.CENTER);
        }
        try{
            left.setIcon(Image.createImage("/images/left.png"));
            right.setIcon(Image.createImage("/images/right.png"));
        }catch(Exception ex){}
        finally{
            left.setText(null);
            right.setText(null);
        }
        anLabel = new AnimatedLabel();
        anLabel.setFocus(true);
        anLabel.setTickerEnabled(true);
        anLabel.startTicker(UIManager.getInstance().getLookAndFeel().getTickerSpeed(), true);
        anLabel.setAlignment(Label.CENTER);

        this.addComponent(BorderLayout.WEST, left);
        this.addComponent(BorderLayout.EAST, right);
        this.addComponent(BorderLayout.CENTER, anLabel);
        if (model.getSize()>0){
            setSelectedIndex(0);
        }
        gr.setId(-1);
    }
    public void actionPerformed(final ActionEvent ev){
        if (!searchMode){
            Display.getInstance().callSerially(new Runnable(){
                public void run(){
                    int curr = model.getSelectedIndex();
                    if (ev.getSource()==left){
                        if (curr>0 ) setSelectedIndex(curr-1);
                        else         setSelectedIndex(size()-1);
                    }else{
                        if (curr<(size()-1) ) setSelectedIndex(curr+1);
                        else                  setSelectedIndex(0);
                    }
                }
            });
        }
    }
    public int size(){
        return model.getSize();
    }
    public void addItem(Object o){
        model.addItem(o);
    }
    public int getSelectedIndex(){
        return model.getSelectedIndex();
    }
    public void setSelectedIndex(int in){
        model.setSelectedIndex(in);
        anLabel.setText(""+(in+1)+". "+model.getItemAt(in).toString());
    }
    public Object getSelectedItem(){
        if (searchMode) return gr;
        else            return model.getItemAt(model.getSelectedIndex());
    }
    public void addSelectionListener(SelectionListener l){
        model.addSelectionListener(l);

    }
    public void removeSelectionListener(SelectionListener l){
        model.removeSelectionListener(l);
    }
    public ListModel getModel(){
        return model;
    }

}
