/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2009 Toporov Konstantin. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 3 only ("GPL")  (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.gnu.org/copyleft/gpl.html  See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * You should include file containing license in each project.
 * If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by
 * the GPL Version 3, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [GPL Version 3] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under the GPL Version 3 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 3 code and therefore, elected the GPL
 * Version 3 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2010 Toporov Konstantin.
 */
package org.diacalc.mobile.components;

/**
 *
 * @author Toporov Konstantin <www.diacalc.org>
 */

import com.sun.lwuit.*;
import com.sun.lwuit.list.*;
import com.sun.lwuit.events.*;
import com.sun.lwuit.layouts.*;
import org.diacalc.mobile.products.*;
import org.diacalc.mobile.maths.*;
import org.diacalc.mobile.DCJmobile;
import com.sun.lwuit.impl.midp.VirtualKeyboard;

public class ProductEditor extends Container implements ActionListener{
      private TextField fldName;
      private TextField fldProt;
      private TextField fldFat;
      private TextField fldCarb;
      private TextField fldGI;
      private TextField fldWeight;
      private ComboBox group;
      private int usage = 0;

      private TextField createTxtFld(){
          TextField fld = new TextField("");
          //fld.setConstraint(TextField.DECIMAL);
          fld.setInputMode("1+2");
          fld.setInputModeOrder(new String[]{"1+2"});
          fld.addActionListener(ProductEditor.this);
          return fld;
      }
      public ProductEditor(ProductInBase pr,DCJmobile master){
          super(new BoxLayout(BoxLayout.Y_AXIS));

          fldName = new TextField("");
          fldProt = createTxtFld();
          fldProt.setHint("Белки");
          fldFat = createTxtFld();
          fldFat.setHint("Жиры");
          fldCarb = createTxtFld();
          fldCarb.setHint("Угл.");
          fldGI = createTxtFld();
          fldGI.setHint("ГИ");
          fldWeight = createTxtFld();
          fldWeight.setHint("вес г.");
          group = new ComboBox(new DefaultListModel(master.getGroups()));
          usage = pr.getUsage();

          addComponent(fldName);
          addComponent(Str.createPair("Б:",fldProt,30));
          addComponent(Str.createPair("Ж:",fldFat,30));
          addComponent(Str.createPair("У:",fldCarb,30));
          addComponent(Str.createPair("ГИ:",fldGI,30));
          addComponent(Str.createPair("Вес:",fldWeight,30));
          addComponent(group);
          int pos = -1;
          for(int i=0;i<master.getGroups().size();i++){
              if ( ((ProductGroup)master.getGroups().elementAt(i)).getId()==pr.getOwner() ){
                  pos = i;
                  break;
              }
          }
          if (pos>=0) group.setSelectedIndex(pos);

          fldProt.setNextFocusUp(fldName);
          fldProt.setNextFocusDown(fldFat);

          fldFat.setNextFocusUp(fldProt);
          fldFat.setNextFocusDown(fldCarb);

          fldCarb.setNextFocusUp(fldFat);
          fldCarb.setNextFocusDown(fldGI);

          fldGI.setNextFocusUp(fldCarb);
          fldGI.setNextFocusDown(fldWeight);

          fldWeight.setNextFocusUp(fldGI);
          fldWeight.setNextFocusDown(group);

          group.setNextFocusUp(fldWeight);
          group.setNextFocusLeft(fldWeight);
          group.setNextFocusDown(fldName);
          group.setNextFocusRight(fldName);

          fldName.setNextFocusUp(group);
          fldName.setNextFocusDown(fldProt);

          setProduct(pr);
          if (Display.getInstance().isTouchScreenDevice()){
                VirtualKeyboard vk = DCJmobile.createRusTxtKeyboard();
                VirtualKeyboard.bindVirtualKeyboard(fldName, vk);

                VirtualKeyboard vknum = DCJmobile.createNumbersKeyboard();
                VirtualKeyboard.bindVirtualKeyboard(fldProt, vknum);
                VirtualKeyboard.bindVirtualKeyboard(fldFat, vknum);
                VirtualKeyboard.bindVirtualKeyboard(fldCarb, vknum);
                VirtualKeyboard.bindVirtualKeyboard(fldGI, vknum);
                VirtualKeyboard.bindVirtualKeyboard(fldWeight, vknum);
          }
          setScrollableY(true);
      }
      private void setProduct(ProductInBase pr){
          fldName.setText( pr.getName().length()==0?"Новый":pr.getName() );
          fldProt.setText( Str.round(pr.getAllProt(),1)>0f?(""+Str.round(pr.getAllProt(),1)):"" );
          fldFat.setText(Str.round(pr.getAllFat(),1)>0f?(""+Str.round(pr.getAllFat(),1)):"");
          fldCarb.setText(Str.round(pr.getAllCarb(),1)>0f?(""+Str.round(pr.getAllCarb(),1)):"");
          fldGI.setText( pr.getGi()>0?(""+pr.getGi()):"" );
          fldWeight.setText(Str.round(pr.getWeight(),0)>0f?(""+Str.round(pr.getWeight(),0)):"");
      }

      public ProductInBase getProduct(){
          float w = Str.parseString(fldWeight, 0);
          return new ProductInBase(
                  fldName.getText(),
                  100 * Str.parseString(fldProt, 1) / w,
                  100 * Str.parseString(fldFat, 1) / w,
                  100 * Str.parseString(fldCarb, 1) / w,
                  (int)Str.parseString(fldGI, 0),
                  Str.parseString(fldWeight, 0),
                  true,
                  ((ProductGroup)group.getModel().getItemAt(group.getSelectedIndex())).getId(),//owner
                  usage
                  );
      }
      public void actionPerformed(ActionEvent ev){
          if (ev.getSource()==fldGI || ev.getSource()==fldWeight){
              Str.parseString( (TextField)ev.getSource(),0);
          }
          else{
              Str.parseString( (TextField)ev.getSource(),1);
          }
      }

      
  }
